@@ -73,6 +73,7 @@ from xml.etree.ElementTree import fromstring
 
 import click
 from dandischema import models
+from dandischema.digests.dandietag import DandiETag
 from pydantic import BaseModel, Field, PrivateAttr
 import requests
 import tenacity
@@ -83,10 +84,10 @@ from .consts import (
     MAX_CHUNK_SIZE,
     RETRY_STATUSES,
     DandiInstance,
+    EmbargoStatus,
     known_instances,
     known_instances_rev,
 )
-from .core.digests.dandietag import DandiETag
 from .exceptions import NotFoundError, SchemaVersionError
 from .keyring import keyring_lookup
 from .utils import USER_AGENT, check_dandi_version, ensure_datetime, is_interactive
@@ -464,7 +465,15 @@ class DandiAPIClient(RESTFullAPIClient):
         if lazy:
             return RemoteDandiset(self, dandiset_id, version_id)
         else:
-            d = RemoteDandiset._make(self, self.get(f"/dandisets/{dandiset_id}/"))
+            try:
+                d = RemoteDandiset.from_data(
+                    self, self.get(f"/dandisets/{dandiset_id}/")
+                )
+            except requests.HTTPError as e:
+                if e.response.status_code == 404:
+                    raise NotFoundError(f"No such Dandiset: {dandiset_id!r}")
+                else:
+                    raise
             if version_id is not None and version_id != d.version_id:
                 if version_id == DRAFT:
                     return d.for_version(d.draft_version)
@@ -479,11 +488,11 @@ class DandiAPIClient(RESTFullAPIClient):
         version if there is one, otherwise to the draft version.
         """
         for data in self.paginate("/dandisets/"):
-            yield RemoteDandiset._make(self, data)
+            yield RemoteDandiset.from_data(self, data)
 
     def create_dandiset(self, name: str, metadata: Dict[str, Any]) -> "RemoteDandiset":
         """Creates a Dandiset with the given name & metadata"""
-        return RemoteDandiset._make(
+        return RemoteDandiset.from_data(
             self, self.post("/dandisets/", json={"name": name, "metadata": metadata})
         )
 
@@ -514,11 +523,20 @@ class DandiAPIClient(RESTFullAPIClient):
 
     def get_asset(self, asset_id: str) -> "BaseRemoteAsset":
         """
-        Fetch the asset with the given asset ID.  The returned object will not
-        have any information about the Dandiset associated with the asset; for
-        that, the `RemoteDandiset.get_asset()` method must be used instead.
+        Fetch the asset with the given asset ID.  If the given asset does not
+        exist, a `NotFoundError` is raised.
+
+        The returned object will not have any information about the Dandiset
+        associated with the asset; for that, the `RemoteDandiset.get_asset()`
+        method must be used instead.
         """
-        return BaseRemoteAsset._from_metadata(self, self.get(f"/assets/{asset_id}"))
+        try:
+            return BaseRemoteAsset.from_metadata(self, self.get(f"/assets/{asset_id}"))
+        except requests.HTTPError as e:
+            if e.response.status_code == 404:
+                raise NotFoundError(f"No such asset: {asset_id!r}")
+            else:
+                raise
 
 
 class APIBase(BaseModel):
@@ -615,7 +633,13 @@ class RemoteDandiset:
 
     def _get_data(self) -> Dict[str, Any]:
         if self._data is None:
-            self._data = self.client.get(f"/dandisets/{self.identifier}/")
+            try:
+                self._data = self.client.get(f"/dandisets/{self.identifier}/")
+            except requests.HTTPError as e:
+                if e.response.status_code == 404:
+                    raise NotFoundError(f"No such Dandiset: {self.identifier}")
+                else:
+                    raise
         return self._data
 
     @property
@@ -659,6 +683,11 @@ class RemoteDandiset:
         """The name of the registered contact person for the Dandiset"""
         return self._get_data()["contact_person"]
 
+    @property
+    def embargo_status(self) -> EmbargoStatus:
+        """The current embargo status for the Dandiset"""
+        return EmbargoStatus(self._get_data()["embargo_status"])
+
     @property
     def most_recent_published_version(self) -> Optional[Version]:
         """
@@ -679,26 +708,48 @@ class RemoteDandiset:
     @property
     def api_path(self) -> str:
         """
-        The API path (relative to the base endpoint for a Dandi Archive API) at
-        which requests for interacting with the Dandiset itself are made
+        The path (relative to the base endpoint for a Dandi Archive API) at
+        which API requests for interacting with the Dandiset itself are made
         """
         return f"/dandisets/{self.identifier}/"
 
+    @property
+    def api_url(self) -> str:
+        """
+        The URL at which API requests for interacting with the Dandiset itself
+        are made
+        """
+        return self.client.get_url(self.api_path)
+
     @property
     def version_api_path(self) -> str:
         """
-        The API path (relative to the base endpoint for a Dandi Archive API) at
-        which requests for interacting with the version in question of the
+        The path (relative to the base endpoint for a Dandi Archive API) at
+        which API requests for interacting with the version in question of the
         Dandiset are made
         """
         return f"/dandisets/{self.identifier}/versions/{self.version_id}/"
 
-    @classmethod
-    def _make(cls, client: "DandiAPIClient", data: Dict[str, Any]) -> "RemoteDandiset":
+    @property
+    def version_api_url(self) -> str:
         """
-        Construct a `RemoteDandiset` instance from a `dict` returned from the
-        API.  If the ``"most_recent_published_version"`` field is set, use that
-        as the Dandiset's version; otherwise, use ``"draft_version"``.
+        The URL at which API requests for interacting with the version in
+        question of the Dandiset are made
+        """
+        return self.client.get_url(self.version_api_path)
+
+    @classmethod
+    def from_data(
+        cls, client: "DandiAPIClient", data: Dict[str, Any]
+    ) -> "RemoteDandiset":
+        """
+        Construct a `RemoteDandiset` instance from a `DandiAPIClient` and a
+        `dict` of raw string fields in the same format as returned by the API.
+        If the ``"most_recent_published_version"`` field is set, that is used
+        as the Dandiset's version; otherwise, ``"draft_version"`` is used.
+
+        This is a low-level method that non-developers would normally only use
+        when acquiring data using means outside of this library.
         """
         if data.get("most_recent_published_version") is not None:
             version = Version.parse_obj(data["most_recent_published_version"])
@@ -708,14 +759,6 @@ class RemoteDandiset:
             client=client, identifier=data["identifier"], version=version, data=data
         )
 
-    def _mkasset(self, data: Dict[str, Any]) -> "RemoteAsset":
-        return RemoteAsset(
-            client=self.client,
-            dandiset_id=self.identifier,
-            version_id=self.version_id,
-            **data,
-        )
-
     def json_dict(self) -> Dict[str, Any]:
         """
         Convert to a JSONable `dict`, omitting the ``client`` attribute and
@@ -739,31 +782,47 @@ class RemoteDandiset:
         before, but the cached version data is internally cleared and may be
         different upon subsequent access.
         """
-        self._data = self.client.get(f"/dandisets/{self.identifier}/")
+        self._data = None
+        self._get_data()
         # Clear _version so it will be refetched the next time it is accessed
         self._version = None
 
     def get_versions(self) -> Iterator[Version]:
         """Returns an iterator of all available `Version`\\s for the Dandiset"""
-        for v in self.client.paginate(f"{self.api_path}versions/"):
-            yield Version.parse_obj(v)
+        try:
+            for v in self.client.paginate(f"{self.api_path}versions/"):
+                yield Version.parse_obj(v)
+        except requests.HTTPError as e:
+            if e.response.status_code == 404:
+                raise NotFoundError(f"No such Dandiset: {self.identifier!r}")
+            else:
+                raise
 
     def get_version(self, version_id: str) -> Version:
         """
         Get information about a given version of the Dandiset.  If the given
-        version does not exist, a `requests.HTTPError` is raised with a 404
-        status code.
+        version does not exist, a `NotFoundError` is raised.
         """
-        return Version.parse_obj(
-            self.client.get(f"/dandisets/{self.identifier}/versions/{version_id}/info")
-        )
+        try:
+            return Version.parse_obj(
+                self.client.get(
+                    f"/dandisets/{self.identifier}/versions/{version_id}/info"
+                )
+            )
+        except requests.HTTPError as e:
+            if e.response.status_code == 404:
+                raise NotFoundError(
+                    f"No such version: {version_id!r} of Dandiset {self.identifier}"
+                )
+            else:
+                raise
 
     def for_version(self, version_id: Union[str, Version]) -> "RemoteDandiset":
         """
         Returns a copy of the `RemoteDandiset` with the `version` attribute set
         to given `Version` object or the `Version` with the given version ID.
-        If a version ID given and the version does not exist, a
-        `requests.HTTPError` is raised with a 404 status code.
+        If a version ID given and the version does not exist, a `NotFoundError`
+        is raised.
         """
         if isinstance(version_id, str):
             version_id = self.get_version(version_id)
@@ -777,7 +836,7 @@ class RemoteDandiset:
     def delete(self) -> None:
         """
         Delete the Dandiset from the server.  Any further access of the
-        instance's data attributes afterwards will result in a 404.
+        instance's data attributes afterwards will result in a `NotFoundError`.
         """
         self.client.delete(self.api_path)
         self._data = None
@@ -795,7 +854,13 @@ class RemoteDandiset:
         Fetch the metadata for this version of the Dandiset as an unprocessed
         `dict`
         """
-        return cast(Dict[str, Any], self.client.get(self.version_api_path))
+        try:
+            return cast(Dict[str, Any], self.client.get(self.version_api_path))
+        except requests.HTTPError as e:
+            if e.response.status_code == 404:
+                raise NotFoundError(f"No such asset: {self}")
+            else:
+                raise
 
     def set_metadata(self, metadata: models.Dandiset) -> None:
         """
@@ -821,7 +886,15 @@ class RemoteDandiset:
         lgr.debug("Waiting for Dandiset %s to complete validation ...", self.identifier)
         start = time()
         while time() - start < max_time:
-            r = self.client.get(f"{self.version_api_path}info/")
+            try:
+                r = self.client.get(f"{self.version_api_path}info/")
+            except requests.HTTPError as e:
+                if e.response.status_code == 404:
+                    raise NotFoundError(
+                        f"No such version: {self.version_id!r} of Dandiset {self.identifier}"
+                    )
+                else:
+                    raise
             if "status" not in r:
                 # Running against older version of dandi-api that doesn't
                 # validate
@@ -848,31 +921,69 @@ class RemoteDandiset:
             Version.parse_obj(self.client.post(f"{self.version_api_path}publish/"))
         )
 
-    def get_assets(self) -> Iterator["RemoteAsset"]:
-        """Returns an iterator of all assets in this version of the Dandiset"""
-        for a in self.client.paginate(f"{self.version_api_path}assets/"):
-            yield self._mkasset(a)
+    def get_assets(self, order: Optional[str] = None) -> Iterator["RemoteAsset"]:
+        """
+        Returns an iterator of all assets in this version of the Dandiset.
+
+        Assets can be sorted by a given field by passing the name of that field
+        as the ``order`` parameter.  The accepted field names are
+        ``"created"``, ``"modified"``, and ``"path"``.  Prepend a hyphen to the
+        field name to reverse the sort order.
+        """
+        try:
+            for a in self.client.paginate(
+                f"{self.version_api_path}assets/", params={"order": order}
+            ):
+                yield RemoteAsset.from_data(self, a)
+        except requests.HTTPError as e:
+            if e.response.status_code == 404:
+                raise NotFoundError(
+                    f"No such version: {self.version_id!r} of Dandiset {self.identifier}"
+                )
+            else:
+                raise
 
     def get_asset(self, asset_id: str) -> "RemoteAsset":
         """
         Fetch the asset in this version of the Dandiset with the given asset
-        ID.  If the given asset does not exist, a `requests.HTTPError` is
-        raised with a 404 status code.
+        ID.  If the given asset does not exist, a `NotFoundError` is raised.
         """
-        metadata = self.client.get(f"{self.version_api_path}assets/{asset_id}/")
+        try:
+            metadata = self.client.get(f"{self.version_api_path}assets/{asset_id}/")
+        except requests.HTTPError as e:
+            if e.response.status_code == 404:
+                raise NotFoundError(f"No such asset: {asset_id!r} for {self}")
+            else:
+                raise
         asset = self.get_asset_by_path(metadata["path"])
         asset._metadata = metadata
         return asset
 
-    def get_assets_with_path_prefix(self, path: str) -> Iterator["RemoteAsset"]:
+    def get_assets_with_path_prefix(
+        self, path: str, order: Optional[str] = None
+    ) -> Iterator["RemoteAsset"]:
         """
         Returns an iterator of all assets in this version of the Dandiset whose
         `~RemoteAsset.path` attributes start with ``path``
+
+        Assets can be sorted by a given field by passing the name of that field
+        as the ``order`` parameter.  The accepted field names are
+        ``"created"``, ``"modified"``, and ``"path"``.  Prepend a hyphen to the
+        field name to reverse the sort order.
         """
-        for a in self.client.paginate(
-            f"{self.version_api_path}assets/", params={"path": path}
-        ):
-            yield self._mkasset(a)
+        try:
+            for a in self.client.paginate(
+                f"{self.version_api_path}assets/",
+                params={"path": path, "order": order},
+            ):
+                yield RemoteAsset.from_data(self, a)
+        except requests.HTTPError as e:
+            if e.response.status_code == 404:
+                raise NotFoundError(
+                    f"No such version: {self.version_id!r} of Dandiset {self.identifier}"
+                )
+            else:
+                raise
 
     def get_asset_by_path(self, path: str) -> "RemoteAsset":
         """
@@ -992,6 +1103,7 @@ class RemoteDandiset:
                         "algorithm": "dandi:dandi-etag",
                         "value": filetag,
                     },
+                    "dandiset": self.identifier,
                 },
             )
         except requests.HTTPError as e:
@@ -1072,18 +1184,20 @@ class RemoteDandiset:
         yield {"status": "producing asset"}
         if replace_asset is not None:
             lgr.debug("%s: Replacing pre-existing asset")
-            a = self._mkasset(
+            a = RemoteAsset.from_data(
+                self,
                 self.client.put(
                     replace_asset.api_path,
                     json={"metadata": asset_metadata, "blob_id": blob_id},
-                )
+                ),
             )
         else:
-            a = self._mkasset(
+            a = RemoteAsset.from_data(
+                self,
                 self.client.post(
                     f"{self.version_api_path}assets/",
                     json={"metadata": asset_metadata, "blob_id": blob_id},
-                )
+                ),
             )
         lgr.info("%s: Asset successfully uploaded", asset_path)
         yield {"status": "done", "asset": a}
@@ -1125,9 +1239,16 @@ class BaseRemoteAsset(APIBase):
         return f"{self.client._instance_id}:assets/{self.identifier}"
 
     @classmethod
-    def _from_metadata(
+    def from_metadata(
         self, client: "DandiAPIClient", metadata: Dict[str, Any]
     ) -> "BaseRemoteAsset":
+        """
+        Construct a `BaseRemoteAsset` instance from a `DandiAPIClient` and a
+        `dict` of raw asset metadata.
+
+        This is a low-level method that non-developers would normally only use
+        when acquiring data using means outside of this library.
+        """
         return BaseRemoteAsset(
             client=client,
             identifier=metadata["identifier"],
@@ -1139,11 +1260,19 @@ class BaseRemoteAsset(APIBase):
     @property
     def api_path(self) -> str:
         """
-        The API path (relative to the base endpoint for a Dandi Archive API) at
-        which requests for interacting with the asset itself are made
+        The path (relative to the base endpoint for a Dandi Archive API) at
+        which API requests for interacting with the asset itself are made
         """
         return f"/assets/{self.identifier}/"
 
+    @property
+    def api_url(self) -> str:
+        """
+        The URL at which API requests for interacting with the asset itself are
+        made
+        """
+        return self.client.get_url(self.api_path)
+
     @property
     def base_download_url(self) -> str:
         """
@@ -1164,7 +1293,13 @@ class BaseRemoteAsset(APIBase):
         if self._metadata is not None:
             return self._metadata
         else:
-            return cast(Dict[str, Any], self.client.get(self.api_path))
+            try:
+                return cast(Dict[str, Any], self.client.get(self.api_path))
+            except requests.HTTPError as e:
+                if e.response.status_code == 404:
+                    raise NotFoundError(f"No such asset: {self}")
+                else:
+                    raise
 
     def get_digest(
         self, digest_type: Union[str, models.DigestType] = models.DigestType.dandi_etag
@@ -1287,14 +1422,45 @@ class RemoteAsset(BaseRemoteAsset):
     #: The date at which the asset was last modified
     modified: datetime
 
+    @classmethod
+    def from_data(
+        self,
+        dandiset: RemoteDandiset,
+        data: Dict[str, Any],
+        metadata: Optional[Dict[str, Any]] = None,
+    ) -> "RemoteAsset":
+        """
+        Construct a `RemoteAsset` instance from a `RemoteDandiset`, a `dict` of
+        raw data in the same format as returned by the API's pagination
+        endpoints, and optional raw asset metadata.
+
+        This is a low-level method that non-developers would normally only use
+        when acquiring data using means outside of this library.
+        """
+        return RemoteAsset(
+            client=dandiset.client,
+            dandiset_id=dandiset.identifier,
+            version_id=dandiset.version_id,
+            **data,
+            _metadata=metadata,
+        )
+
     @property
     def api_path(self) -> str:
         """
-        The API path (relative to the base endpoint for a Dandi Archive API) at
-        which requests for interacting with the asset itself are made
+        The path (relative to the base endpoint for a Dandi Archive API) at
+        which API requests for interacting with the asset itself are made
         """
         return f"/dandisets/{self.dandiset_id}/versions/{self.version_id}/assets/{self.identifier}/"
 
+    @property
+    def api_url(self) -> str:
+        """
+        The URL at which API requests for interacting with the asset itself are
+        made
+        """
+        return self.client.get_url(self.api_path)
+
     @property
     def download_url(self) -> str:
         """
@@ -1312,8 +1478,8 @@ class RemoteAsset(BaseRemoteAsset):
 
     def set_raw_metadata(self, metadata: Dict[str, Any]) -> None:
         """
-        Set the metadata for the asset to the given value and update the
-        `RemoteAsset` in place.
+        Set the metadata for the asset on the server to the given value and
+        update the `RemoteAsset` in place.
         """
         try:
             etag = metadata["digest"]["dandi:dandi-etag"]
